////////////////////////////////////////////////////////////////////////
// Bulldozer - get away from the construction site
//----------------------------------------------------------------------
// Discription :	100 Lines contest
// Created		:	Erik Zinger (kasimir) : 2008-06-07 
// E-mail		:	kasimir_1985@yahoo.de
//----------------------------------------------------------------------
////////////////////////////////////////////////////////////////////////

#define PRAGMA_PATH "./source";

VECTOR temp;

#define cam_tilt_min -60
#define cam_tilt_max -25

#define cam_dist_min 256		//Zoom max
#define cam_dist_max 1024		//zoom min

var camera_dist = cam_dist_max;


ENTITY* lvl_ptr;


SOUND* music1_wav = "relax_02.wav";

SOUND* horn_wav = "trainhorn.wav";
SOUND* engine_wav = "engine.wav";

var engine_handle;


var positions[8] = {237, 238, 233, 234, 235, 236, 239, 240};		//Vertex numbers
var wheel[6];																		//wheel pointer
var index = 0;


#define speed 15

#define force_left 	skill10
#define force_right	skill11


function restart_lvl ()
{
	if (lvl_ptr.skill1 == 0) {level_load ("level1.WMB");}
	if (lvl_ptr.skill1 == 1) {level_load ("level2.WMB");}
	if (lvl_ptr.skill1 == 2) {level_load ("level3.WMB");}
}


BMAP* startscreen_map = "startscreen.bmp"; 
BMAP* credits_map = "credits.bmp";

PANEL* start_pan =
{
	bmap = startscreen_map;
	flags = visible;
}


function main ()
{
	video_mode = 8;
	video_screen = 1;

	level_load ("level1.WMB");
	wait (2);

	ph_setgravity (vector (0, 0, -386));

	while (!key_any) {wait (1);}

	on_r = restart_lvl;

	reset (start_pan, VISIBLE);

	snd_loop (music1_wav, 50, 0);
	engine_handle = snd_loop (engine_wav, 50, 0);

	while (1)
	{
		camera.pan -= 50 * mouse_force.x * time_step * mouse_right;
		camera.tilt = clamp (ang (camera.tilt + 50 * mouse_force.y * time_step * mouse_right), cam_tilt_min, cam_tilt_max);

		camera_dist = clamp (camera_dist - 2 * mickey.z * time_step, cam_dist_min, cam_dist_max);

		camera.x = player.x - camera_dist * cos (camera.pan) * cos (camera.tilt);
		camera.y = player.y - camera_dist * sin (camera.pan) * cos (camera.tilt);
		camera.z = player.z - camera_dist * sin (camera.tilt);	

		player.force_left  = clamp (key_cuu - key_cud + (key_cur - key_cul) * (1 - 2 * key_cud), -1, 1);
		player.force_right = clamp (key_cuu - key_cud + (key_cul - key_cur) * (1 - 2 * key_cud), -1, 1);

		phcon_setmotor (wheel[1], vector (-player.force_left * speed, 15000, 0), nullvector, nullvector); 
		phcon_setmotor (wheel[3], vector (-player.force_left * speed, 15000, 0), nullvector, nullvector); 
		phcon_setmotor (wheel[5], vector (-player.force_left * speed, 15000, 0), nullvector, nullvector); 		
		phcon_setmotor (wheel[0], vector (-player.force_right * speed, 15000, 0), nullvector, nullvector); 
		phcon_setmotor (wheel[2], vector (-player.force_right * speed, 15000, 0), nullvector, nullvector); 
		phcon_setmotor (wheel[4], vector (-player.force_right * speed, 15000, 0), nullvector, nullvector); 

		snd_tune (engine_handle, 50, 30 + 30 * minv (key_cuu + key_cud + key_cul + key_cur, 1), 0);

		wait(1);
	}
}


function dozers_exhaust ()
{
	set (my, PASSABLE | UNLIT | BRIGHT);
	while (1)
	{
		vec_for_vertex (temp.x, player, 241);		//get position
		vec_set (my.x, temp.x);
		vec_set (my.pan, vector (player.pan, 
			player.tilt + 1 * sin (100 * total_ticks) * clamp (key_cuu + key_cud + key_cul + key_cur, 0.5, 1), 
			player.roll + 1 * cos (100 * total_ticks) * clamp (key_cuu + key_cud + key_cul + key_cur, 0.5, 1)));

		wait (1);
	}
}


function dozers_track ()
{
	set (my, PASSABLE | UNLIT | BRIGHT);

	my.skill1 = index;
	index += 1;

	while (1)
	{
		vec_for_vertex (temp.x, player, positions[my.skill1]);		//get position
		vec_set (my.x, temp.x);

		vec_set (my.pan, player.pan);

		//anim track:
		my.u += 15 * player.force_left * time_step * (my.skill1 - 6) +  15 * player.force_right * time_step * (1 - my.skill1 + 6);

		wait (1);
	}
}


function dozers_tyre ()
{
	set(my,UNLIT | BRIGHT); 

	if (index < 2) {set (my, INVISIBLE); }				//middle wheels not visible
	if (fraction (index / 2)) {my.pan = 180; }		//right or left?! -> rotate

	phent_settype (my, PH_RIGID, PH_SPHERE);
	phent_setmass (my, 10, PH_SPHERE);
	phent_setgroup (my, 2);
	phent_setfriction (my, 100);
	phent_setdamping (my, 0, 0);
	phent_setelasticity (my, 0, 0);

	wheel[index] = phcon_add (PH_HINGE, player, my);
	phcon_setparams1 (wheel[index], my.x, vector (0, 1, 0), nullvector);
	phcon_setparams2 (wheel[index], vector (-360, 360, 0), nullvector, nullvector);
}


action dozer ()
{
	player = my;

	phent_settype (my, PH_RIGID, PH_BOX);
	phent_setmass (my, 2000, PH_BOX);
	phent_setgroup (my, 2);
	phent_setfriction (my, 10);
	phent_setdamping (my, 0, 0);
	phent_setelasticity (my, 30, 10);

	for (index = 0; index < 6; index++)
	{
		vec_for_vertex (temp.x, my, positions[index]);
		ent_create ("wheel.mdl", temp.x, dozers_tyre);
	}

	ent_create ("track.mdl", temp, dozers_track);
	ent_create ("track.mdl", temp, dozers_track);

	ent_create ("exhaust.mdl", temp, dozers_exhaust);
}


action crate ()
{
	phent_settype (my, PH_RIGID, PH_BOX);
	phent_setmass (my, 500, PH_BOX);
	phent_setfriction (my, 70);
	phent_setdamping (my, 50, 50);
	phent_setelasticity (my, 30, 30);
}


action lvl_target ()			//"exit" load new level
{
	lvl_ptr = me;

	while (vec_dist (my.x, player.x) > 96) {my.pan += 5 * time_step; wait (1); }

	my.skill2 = snd_play (horn_wav, 100,0);
	
	while (snd_playing (my.skill2)) {wait(1); }
	
	if (my.skill1 == 0) 
	{	
		level_load ("level2.WMB"); 
	}else{
		if (my.skill1 == 1) 
		{
			level_load ("level3.WMB"); 
		}else{
			if (my.skill1 == 2) 					//show credtis - exit game
			{
				start_pan.bmap = credits_map;
				set (start_pan, VISIBLE);
				while (key_any) { wait (1); }
				while (!key_any) { wait (1); }
				sys_exit ("Thank you for playing"); 
			}
		}
	}
}